import os


def main():
	# Arcmap stuff
	mxd = arcpy.mapping.MapDocument("CURRENT")
	df = arcpy.mapping.ListDataFrames(mxd, "*")[0]
	
	# Inputs
	inputLayer = arcpy.GetParameter(0)
	usInvert = arcpy.GetParameterAsText(1)
	dsInvert = arcpy.GetParameterAsText(2)
	newField = 'wrongDir'  # new field with wrong direction flag
	loc = os.path.dirname(inputLayer.dataSource)  # location of file
	
	# Check for adverse gradient
	arcpy.AddMessage('\nProcessing.. Looking for adverse gradients')
	arcpy.AddField_management(inputLayer, newField, "DOUBLE", 15, 5, "#", "#", "#", "#", "#")
	codeblock = """def wrongDir(usIL, dsIL):
		if usIL < dsIL:
			return 1
		else:
			return 0"""
	expression = "wrongDir(!{0}!, !{1}!)".format(usInvert, dsInvert)
	arcpy.CalculateField_management(inputLayer, newField, expression, 'PYTHON_9.3', codeblock)
	
	# Reverse line direction if new field has value 1
	arcpy.AddMessage('Processing.. Reversing line directions')
	searchCursor = ['SHAPE@', 'OID@', newField]
	outLayer = []
	for row in arcpy.da.SearchCursor(inputLayer, searchCursor):
		outFeature = []
		if row[-1] == 1:
			arcpy.AddMessage('FID{0} is being reversed.'.format(row[1]))
			for vertices in row[0]:
				for vertice in vertices:
					outFeature.insert(0, [vertice.X, vertice.Y])
		else:
			arcpy.AddMessage('FID{0} is remaining the same.'.format(row[1]))
			for vertices in row[0]:
				for vertice in vertices:
					outFeature.append([vertice.X, vertice.Y])
		outLayer.append(outFeature)
		
	# create new polyline layer with new directions
	arcpy.AddMessage('Processing.. Creating new polylines')
	polylines = []
	for feature in outLayer:
		polylines.append(
			arcpy.Polyline(
				arcpy.Array([arcpy.Point(*coords) for coords in feature])))
				
	
	# join fields of original layer and remove added fields
	arcpy.AddMessage('Processing.. Joining attributes')
	outFile = arcpy.CopyFeatures_management(polylines, os.path.join(loc, 'Corrected.shp'))
	arcpy.JoinField_management(outFile, 'FID', inputLayer, 'FID')
	arcpy.DeleteField_management(outFile, "ID")
	arcpy.DeleteField_management(inputLayer, newField)
	
	# update upstream and downstream invert levels
	arcpy.AddMessage('Processing.. Switching Upstream and Downstream inverts.')
	arcpy.AddField_management(outFile, "newUsIL", "DOUBLE", 15, 5, "#", "#", "#", "#", "#")
	arcpy.AddField_management(outFile, "newDsIL", "DOUBLE", 15, 5, "#", "#", "#", "#", "#")	
	codeblock_usIL = """def getIL(input, usIL, dsIL):
		if input == 1:
			return dsIL
		else:
			return usIL"""
	codeblock_dsIL = """def getIL(input, usIL, dsIL):
		if input == 1:
			return usIL
		else:
			return dsIL"""
	expression = "getIL(!{0}!, !{1}!, !{2}!)".format(newField, usInvert, dsInvert)
	arcpy.CalculateField_management(outFile, 'newUsIL', expression, 'PYTHON_9.3', codeblock_usIL)
	arcpy.CalculateField_management(outFile, 'newDsIL', expression, 'PYTHON_9.3', codeblock_dsIL)
	# copy dummy fields to original fields and delete dummy fields
	arcpy.CalculateField_management(outFile, usInvert, '!newUsIL!', 'PYTHON_9.3', codeblock_usIL)
	arcpy.CalculateField_management(outFile, dsInvert, '!newDsIL!', 'PYTHON_9.3', codeblock_usIL)
	arcpy.DeleteField_management(outFile, "newUsIL")
	arcpy.DeleteField_management(outFile, "newDsIL")
	arcpy.DeleteField_management(outFile, newField)
	
	# add layer to map
	newlayer = arcpy.mapping.Layer(os.path.join(loc, 'Corrected.shp'))
	arcpy.mapping.AddLayer(df, newlayer, "TOP")
	arcpy.RefreshActiveView()
	arcpy.RefreshTOC()
	
if __name__ == "__main__":
	main()
