import os


def main():
	# Arcmap stuff
	mxd = arcpy.mapping.MapDocument("CURRENT")
	df = arcpy.mapping.ListDataFrames(mxd, "*")[0]
	
	# Inputs
	inputLayer = arcpy.GetParameter(0)
	usInvert = arcpy.GetParameterAsText(1)
	dsInvert = arcpy.GetParameterAsText(2)
	newField = 'wrongDir'  # new field with wrong direction flag
	inputDS = os.path.splitext(inputLayer.dataSource)[0]  # input DataSource
	
	# Check for adverse gradient
	arcpy.AddMessage('\nProcessing.. Looking for adverse gradients')
	arcpy.AddField_management(inputLayer, newField, "DOUBLE", 15, 5, "#", "#", "#", "#", "#")
	codeblock = """def wrongDir(usIL, dsIL):
		if usIL < dsIL:
			return 1
		else:
			return 0"""
	expression = "wrongDir(!{0}!, !{1}!)".format(usInvert, dsInvert)
	arcpy.CalculateField_management(inputLayer, newField, expression, 'PYTHON_9.3', codeblock)
	
	# Map Fields of input layer
	fms = arcpy.FieldMappings()
	fm =arcpy.FieldMap()
	for i, field in enumerate(arcpy.ListFields(inputLayer)):
		if i > 1:
			fm = arcpy.FieldMap()
			fm.addInputField(inputLayer.dataSource, field.name)
			fms.addFieldMap(fm)
	
	# Reverse line direction if new field has value 1
	arcpy.AddMessage('Processing.. Reversing line directions')
	searchCursor = ['SHAPE@', 'OID@', newField]
	outLayer = []
	for row in arcpy.da.SearchCursor(inputLayer, searchCursor):
		outFeature = []
		if row[-1] == 1:
			arcpy.AddMessage('FID{0} is being reversed.'.format(row[1]))
			for vertices in row[0]:
				for vertice in vertices:
					outFeature.insert(0, [vertice.X, vertice.Y])
		else:
			arcpy.AddMessage('FID{0} is remaining the same.'.format(row[1]))
			for vertices in row[0]:
				for vertice in vertices:
					outFeature.append([vertice.X, vertice.Y])
		outLayer.append(outFeature)
		
	# create new polyline layer with new directions
	arcpy.AddMessage('\nProcessing.. Creating new polylines')
	polylines = []
	for feature in outLayer:
		polylines.append(
			arcpy.Polyline(
				arcpy.Array([arcpy.Point(*coords) for coords in feature])))
				
	
	# join fields of original layer
	arcpy.AddMessage('\nProcessing.. Joining attributes')
	outFile = arcpy.CopyFeatures_management(polylines, os.path.join(os.path.dirname(inputDS), '_Corrected.shp'))
	polyLineFeature = arcpy.MakeFeatureLayer_management(outFile, "feature_layer")
	inputTableView = arcpy.MakeTableView_management(inputLayer, "table_view")
	arcpy.AddJoin_management(polyLineFeature, 'FID', inputTableView, 'FID')
	arcpy.CreateFileGDB_management(os.path.dirname(inputDS), 'temp')
	outFile = arcpy.FeatureClassToFeatureClass_conversion(polyLineFeature, os.path.join(os.path.dirname(inputDS), 'temp.gdb'), '_Corrected')
	for i, field in enumerate(arcpy.ListFields(outFile)):
		if i > 2:
			try:
				length = len(inputLayer.name)
				fname = field.name[length + 1:]
				arcpy.AlterField_management(outFile, field.name, fname)
				arcpy.AddMessage('{}'.format(fname))
			except:
				continue
	outFile2 = arcpy.FeatureClassToFeatureClass_conversion(outFile, os.path.join(os.path.dirname(inputDS)), os.path.basename(inputDS) + '_Corrected.shp', '#', fms)
	arcpy.Delete_management(polyLineFeature)
	del outFile
	del polyLineFeature
	del inputTableView
	arcpy.Delete_management(os.path.join(os.path.dirname(inputDS), '_Corrected.shp'))
	arcpy.Delete_management(os.path.join(os.path.dirname(inputDS), 'temp.gdb'))
	
	# update upstream and downstream invert levels
	arcpy.AddMessage('\nProcessing.. Switching Upstream and Downstream inverts.')
	arcpy.AddField_management(outFile2, "newUsIL", "DOUBLE", 15, 5, "#", "#", "#", "#", "#")
	arcpy.AddField_management(outFile2, "newDsIL", "DOUBLE", 15, 5, "#", "#", "#", "#", "#")	
	codeblock_usIL = """def getIL(input, usIL, dsIL):
		if input == 1:
			return dsIL
		else:
			return usIL"""
	codeblock_dsIL = """def getIL(input, usIL, dsIL):
		if input == 1:
			return usIL
		else:
			return dsIL"""
	expression = "getIL(!{0}!, !{1}!, !{2}!)".format(newField, usInvert, dsInvert)
	arcpy.CalculateField_management(outFile2, 'newUsIL', expression, 'PYTHON_9.3', codeblock_usIL)
	arcpy.CalculateField_management(outFile2, 'newDsIL', expression, 'PYTHON_9.3', codeblock_dsIL)
	# copy dummy fields to original fields and delete dummy fields
	arcpy.CalculateField_management(outFile2, usInvert, '!newUsIL!', 'PYTHON_9.3', codeblock_usIL)
	arcpy.CalculateField_management(outFile2, dsInvert, '!newDsIL!', 'PYTHON_9.3', codeblock_usIL)
	arcpy.DeleteField_management(outFile2, "newUsIL")
	arcpy.DeleteField_management(outFile2, "newDsIL")
	arcpy.DeleteField_management(outFile2, newField)
	
	# add layer to map
	newlayer = arcpy.mapping.Layer(inputDS + '_Corrected.shp')
	arcpy.mapping.AddLayer(df, newlayer, "TOP")
	arcpy.RefreshActiveView()
	arcpy.RefreshTOC()
	
if __name__ == "__main__":
	main()
