import os
import sys

# ______________________________________________________________________________________________________________________
# Functions

def import_empty(data_frame, tf_type, output_prefix, geom_type, geom_ext, gis_fol, empty_folder, field_len):
	warning_status = False
	error_status = False
	
	# if number of input fields is 10 or more, need to use nwke
	if tf_type == '1d_nwk':
		if field_len >= 10:
			tf_type2 = '1d_nwke'
			empty_basename = '{0}_empty{1}.shp'.format(tf_type2, geom_ext)
		else:
			empty_basename = '{0}_empty{1}.shp'.format(tf_type, geom_ext)
	else:
		empty_basename = '{0}_empty{1}.shp'.format(tf_type, geom_ext)
	empty_file = os.path.join(empty_folder, empty_basename)
	out_basename = '{0}_{1}{2}.shp'.format(tf_type, output_prefix, geom_ext)
	out_file = os.path.join(gis_fol, out_basename)
	temp_file = os.path.join(gis_fol, '_temp.shp')
	
	if os.path.isfile(empty_file):
		if os.path.isfile(out_file):
			error_status = True
			return_message = '{0} already exists.'.format(out_basename)
			return error_status, warning_status, return_message, out_file, out_basename, temp_file
		else:
			sr = arcpy.Describe(empty_file).spatialReference
			arcpy.CreateFeatureclass_management(gis_fol, '_temp.shp', geom_type, empty_file, '#', '#', sr)
			try:
				arcpy.CreateFeatureclass_management(gis_fol, out_basename, geom_type, empty_file, '#', '#', sr)
			except Exception:  # file name most likely has 'invalid characters'. Create file as temp then rename.
				arcpy.CreateFeatureclass_management(gis_fol, 'temp.shp', geom_type, empty_file, '#', '#', sr)
				arcpy.Rename_management(os.path.join(gis_fol, 'temp.shp'), temp_file)
			#newlayer = arcpy.mapping.Layer(out_file)
			#arcpy.mapping.AddLayer(data_frame, newlayer, "TOP")
			#arcpy.RefreshActiveView()
			#arcpy.RefreshTOC()
			return_message = "Successfully imported {0}".format(out_basename)
			return error_status, warning_status, return_message, out_file, out_basename, temp_file
	else:
		warning_status = True
		return_message = 'Invalid geometry type for {0}... skipping'.format(tf_type)
		return error_status, warning_status, return_message, outFile, out_basename, temp_file
		

def main():
	
	# Get user parameters
	emptyFolder = arcpy.GetParameterAsText(0)
	inputLayer = arcpy.GetParameter(1)
	tfAttributeType = arcpy.GetParameterAsText(2)
	runID = arcpy.GetParameterAsText(3)
	
	# some arcmap stuff:
	mxd = arcpy.mapping.MapDocument("CURRENT")
	df = arcpy.mapping.ListDataFrames(mxd, "*")[0]
	
	# find GIS folder
	gisFolder = emptyFolder[:-6]  # removes last 6 characters (empty\)
	
	# Determine Geometry Type
	if arcpy.Describe(inputLayer).shapeType == 'Polygon':
		geomType = "POLYGON"
		geomExt = '_R'
	elif arcpy.Describe(inputLayer).shapeType == 'Polyline':
		geomType = "POLYLINE"
		geomExt = '_L'
	elif arcpy.Describe(inputLayer).shapeType == 'Point':
		geomType = "POINT"
		geomExt = '_P'
	else:
		arcpy.AddError("\nUnable to recognise input layer geometry type: {0}".format(inputLayer))
		
	# Determine number of fields in input layer
	lenFields = len(arcpy.ListFields(inputLayer)) - 2  # Don't include FID and Shape fields
	
	errorStatus, warningStatus, message, outFile, outBasename, tempFile = import_empty(df, tfAttributeType, runID, geomType, geomExt, gisFolder, emptyFolder, lenFields)
	arcpy.AddMessage('\nProcessing.. Creating TUFLOW Layer {0}'.format(outBasename))
	if errorStatus:
		arcpy.AddError(message)
	if warningStatus:
		arcpy.AddWarning(message)
	
	# find duplicated fields so they can be zero'd out after merge
	field_list = []
	duplicate_fields = []
	outFile_fields = []
	inputLayer_fields = []
	len_outFile = len(arcpy.ListFields(outFile))
	len_inputLayer = len(arcpy.ListFields(inputLayer))
	for field in arcpy.ListFields(outFile):
		field_list.append(field.name.lower())
		outFile_fields.append(field.name)
	for i, field in enumerate(arcpy.ListFields(inputLayer)):
		if field.name != 'Shape':
			inputLayer_fields.append(field.name)
		if i > 1:
			if field.name.lower() not in field_list:
				field_list.append(field.name.lower())
			else:
				duplicate_fields.append(field.name)
				#field_list.append('{0}_1'.format(field.name))
	field_list = outFile_fields + inputLayer_fields

	# Append data and zero out duplicate fields
	arcpy.AddMessage('\nProcessing.. Appending data from Input Layer {0}'.format(inputLayer))
	noTest = ""
	for field in duplicate_fields:
		noTest = "{0}{1} \"{1}\" true false false;".format(noTest, field)
	arcpy.Append_management(inputLayer, outFile, 'NO_TEST', noTest)
	## don't need the below code as the 'noTest' is being defined in the step above which is really nice
	#with arcpy.da.UpdateCursor(outFile, duplicate_fields) as cursor:
	#	for row in cursor:
	#		for i in range(len(duplicate_fields)):
	#			try:
	#				row[i] = ''
	#				cursor.updateRow(row)
	#			except:
	#				row[i] = 0
	#				cursor.updateRow(row)
	
	## Join data
	#arcpy.JoinField_management(outFile, 'FID', inputLayer, 'FID')
	# The single line method above of joining data has been superseded with a far more complicated (and messy) method.
	# The reason is that is the method below is quicker when dealing with large data sets. A trial data set took 8mins with the above method and only 40s with the below method.
	
	fc = arcpy.MakeFeatureLayer_management(outFile, "feature_layer")
	tv = arcpy.MakeTableView_management(inputLayer, "table_view")
	arcpy.AddJoin_management(fc, 'FID', tv, 'FID')
	arcpy.CreateFileGDB_management(gisFolder, 'temp')
	tempFile2 = arcpy.FeatureClassToFeatureClass_conversion(fc, os.path.join(gisFolder, 'temp.gdb'), '_joined')
	arcpy.AddMessage('Processing.. Appending fields from input layer to TUFLOW layer:')
	for i, field in enumerate(arcpy.ListFields(tempFile2)):
		if i > 1 and i < len(arcpy.ListFields(tempFile2)) - 1:
			if i < len_outFile:
				fname = field_list[i]
				#arcpy.AddMessage('i: {0}, file name: {1}, field name: {2}, new field name: {3}'.format(i, inputLayer.name, field.name, fname))
				arcpy.AlterField_management(tempFile2, field.name, fname)
			else:
				fname = field_list[i]
				#arcpy.AddMessage('i: {0}, file name: {1}, field name: {2}, new field name: {3}'.format(i, inputLayer.name, field.name, fname))
				try:
					
					arcpy.AlterField_management(tempFile2, field.name, fname)
					arcpy.AddMessage('{}'.format(fname))
				except:
					fname = '{}_1'.format(fname)
					arcpy.AlterField_management(tempFile2, field.name, fname)
					arcpy.AddMessage('{}'.format(fname))
	tempFile3 = arcpy.FeatureClassToFeatureClass_conversion(tempFile2, gisFolder, '_join.shp')
	arcpy.DeleteField_management(tempFile3, "FID_")
	arcpy.DeleteField_management(tempFile3, "Shape_Leng")
	arcpy.Merge_management([tempFile, tempFile3], outFile, 'FIRST')
	arcpy.Delete_management(os.path.join(gisFolder, 'temp.gdb'))
	arcpy.Delete_management(tempFile)
	arcpy.Delete_management(tempFile3)
	arcpy.Delete_management(fc)
	arcpy.Delete_management(tv)
	del fc
	del tv
	
	# Add layer to map window
	newLayer = arcpy.mapping.Layer(outFile)
	arcpy.mapping.AddLayer(df, newLayer, "TOP")
	arcpy.RefreshActiveView()
	arcpy.RefreshTOC()

# ______________________________________________________________________________________________________________________
# Main Code

	
if __name__ == '__main__':
	main()
