import os
import re # regular expressions for finding patterns
# must be run in foreground
# Created by ES and EM
import arcpy
# ______________________________________________________________________________________________________________________
# functions


def update_revision(old_path, new_rev, add_rev):
    if add_rev:
        # appending rev number to layer before region type in name (P, L, R)
        old_name, old_ext = os.path.splitext(os.path.basename(old_path))
        if old_name[-2:] == "_P" or old_name[-2:] == "_L" or old_name[-2:] == "_R":
            new_lyr_path = os.path.dirname(old_path) + os.sep + old_name[:-2] + "_" + new_rev + old_name[-2:] + old_ext
        else:
            new_lyr_path = os.path.dirname(old_path) + os.sep + old_name + "_" + new_rev + old_ext
        return new_lyr_path
    else:
        # find old rev number and replace with new and return full path
        old_rev = re.findall(r"(\d{3})", old_path)  # \d means a digit, and the {3} means 3 times. () forms groups.
        if old_rev:  # at least one match found
            old_rev = old_rev[-1]  # return last match (should be rev number)
            new_lyr_path = os.path.dirname(old_path) + os.sep
            new_lyr_path += old_rev.join(os.path.basename(old_path).split(old_rev)[:-1]) + new_rev\
                            + os.path.basename(old_path).split(old_rev)[-1]
            return new_lyr_path
        else:
            return None
    
# ______________________________________________________________________________________________________________________
# Main code
    

if __name__ == "__main__":
    # get parameters. general practice is to use camelCase (like TitleCase but with a lowercase start)
    oldLyrs = arcpy.GetParameter(0)  # Feature Layer
    outputRunNum = arcpy.GetParameterAsText(1)  # String, was called Output_Run_Number
    ss = arcpy.GetParameter(2)  # keep as Boolean, doesn't seem to matter in this case though
    addRevNum = arcpy.GetParameter(3)  # Boolean
    
    # get the old path and new path using our updateRevision function defined earlier
    for oldLyr in oldLyrs:
        oldLyrPath = oldLyr.dataSource
        oldLyrName = oldLyr.name
        arcpy.AddMessage("\nUpdating {}...".format(oldLyrName))
        newLyrPath = update_revision(oldLyrPath, outputRunNum, addRevNum)
        # copy to new shp if the new source is valid
        if newLyrPath:  # None will evaluate to False
            newLyrName = os.path.splitext(os.path.basename(newLyrPath))[0]
            try:
                arcpy.CopyFeatures_management(oldLyr, newLyrPath)
            except Exception:  # file name most likely has 'invalid characters'. Create file as temp then rename.
                arcpy.CopyFeatures_management(oldLyr, os.path.join(os.path.dirname(newLyrPath), 'temp'))
                arcpy.Rename_management(os.path.join(os.path.dirname(newLyrPath), 'temp.shp'), newLyrPath)
        else:    
            arcpy.AddError("Failed to update revision for {}.".format(oldLyrName))
            # break
        
        # update datasource of current instance layer and rename
        if arcpy.Exists(newLyrPath):
            mxd = arcpy.mapping.MapDocument("CURRENT")  # ensures we use the layer we're looking at
            currentLyr = arcpy.mapping.ListLayers(mxd, oldLyrName)[0]
            currentLyr.replaceDataSource(oldLyr.workspacePath, "SHAPEFILE_WORKSPACE", newLyrName)
            currentLyr.name = newLyrName
            arcpy.RefreshTOC(), arcpy.RefreshActiveView()
            arcpy.AddMessage("Successfully updated {0} to {1}.".format(oldLyrName, outputRunNum))
            
        # move input lyr to superseded folder
        if ss:
            ssDir = os.path.join(oldLyr.workspacePath, "SS")
            if not os.path.exists(ssDir):
                os.mkdir(ssDir)
            try:
                arcpy.CopyFeatures_management(oldLyrPath, os.path.join(ssDir, oldLyrName + ".shp"))
            except Exception:  # file name most likely has 'invalid characters'. Create file as temp then rename.
                arcpy.CopyFeatures_management(oldLyrPath, os.path.join(ssDir, 'temp'))
                arcpy.Rename_management(os.path.join(ssDir, 'temp.shp'), os.path.join(ssDir, oldLyrName + ".shp"))
            arcpy.Delete_management(oldLyrPath)
