"""
 --------------------------------------------------------
		arcTUFLOW_Library - Arc TUFLOW functionality
        begin                : 2017-01-19
        copyright            : (C) 2017 by Mitchell Smith
        email                : support@tuflow.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
import os
import arcpy
import unicodedata
from arcpy import env

shp_in = arcpy.GetParameterAsText(0)
field_mod = arcpy.GetParameterAsText(1)
new_type = arcpy.GetParameterAsText(2)
f_len_or_prc = arcpy.GetParameterAsText(3)
shp_out = arcpy.GetParameterAsText(4)

# For testing.
#shp_in = r'C:\Perth\Advanced\QGIS\Complete_Model\TUFLOW\model\gis\1d_nwk_M02_culverts_001_L.shp'
#field_mod = 'PBLOCKAGE'
#new_type = 'TEXT'
tmp_t1 = os.path.join(os.path.dirname(shp_in),'tmp1.shp')
tmp_t2 = os.path.join(os.path.dirname(shp_in),'tmp2.shp')
#output_shp = r'C:\Perth\Advanced\QGIS\Complete_Model\TUFLOW\model\gis\1d_nwk_M02_culverts_003_L.shp'
found = False
f_k1 = [] 		# Field names prior to new field
f_k2 = []       # Field names after new field.

# Check if shp_out exists and prompt for overwrite  (to be added)
try:
    arcpy.Delete_management(tmp_t1)
except OSError:
    pass
try:
    arcpy.Delete_management(tmp_t2)
except OSError:
    pass
try:
    arcpy.Delete_management(shp_out)
except OSError:
    pass

# 1. Extract the field names and data types from the table
fields = arcpy.ListFields(shp_in)

# 2. Find the field that needs it's type updated
for f in fields:
	if str(f.name).lower() == field_mod.lower() and not found:
		found = True
		f_k1.append(str(f.name))
		continue
	# Once found append all fields following our mod field so we can delete later.
	if found:
		f_k1.append(str(f.name))
	else:
	# Append all fields preceeding the update field so we can delete them later
		f_k2.append(str(f.name))
f_k2.append(field_mod)

# Test for existence of the field we're wanting to change
if not found:
	print('Field with name: ', field_mod, ' not found. Please check the field exists')
	arcpy.AddError("Field with name: {0} does not exist. Please check.".format(field_mod))

# 3. Create a copy of the shp file that will be our final dataset (and a tmp for editing).
arcpy.Copy_management(shp_in , tmp_t1)
arcpy.Copy_management(shp_in , tmp_t2)

# 4. Delete all fields including and to the right of our column of interest.
arcpy.DeleteField_management(tmp_t1, f_k1)
arcpy.DeleteField_management(tmp_t2, f_k2[2:])

# 5. Add new Field.  (Note. At present this will only accomodate the creation of text fields. Need to add if block based on data type).
arcpy.AddField_management(tmp_t1, field_mod,new_type, field_length=f_len_or_prc)
#arcpy.AddField_management(tmp_t1, unicode(field_mod,'utf-8'), unicode(new_type,'utf-8'), field_length=f_len_or_prc)

# 6. Join the two tables.
arcpy.SpatialJoin_analysis(tmp_t1, tmp_t2, shp_out, 'JOIN_ONE_TO_ONE', 'KEEP_ALL','#', 'ARE_IDENTICAL_TO', '#', '#')

# 7 Tidy up.
try:
    arcpy.Delete_management(tmp_t1)
    arcpy.Delete_management(tmp_t2)
except OSError:
    pass
