""" ---------------------------------------------------------------------------
# import_empties_v2.py
# Created on: 2012-03-30 17:00:27.00000
#   (generated by ArcGIS/ModelBuilder)
# Usage: import_empties_v2 <Points> <Lines> <Regions> <Empty_Shapefile> <Output_Prefix> 
# Description: 
# ---------------------------------------------------------------------------
"""

import os
import arcpy

# ______________________________________________________________________________________________________________________
# Functions


def import_empty(data_frame, tf_type, output_prefix, geom_type, geom_ext, gis_fol, empty_folder):
    warning_status = False
    empty_basename = '{0}_empty{1}.shp'.format(tf_type, geom_ext)
    empty_file = os.path.join(empty_folder, empty_basename)
    out_basename = '{0}_{1}{2}.shp'.format(tf_type, output_prefix, geom_ext)
    out_file = os.path.join(gis_fol, out_basename)
    
    if os.path.isfile(empty_file):
        if os.path.isfile(out_file):
            warning_status = True
            return_message = '{0} already exists... skipping'.format(out_basename)
            return warning_status, return_message
        else:
            try:
                sr = arcpy.Describe(empty_file).spatialReference
                arcpy.CreateFeatureclass_management(gis_fol, out_basename, geom_type, empty_file, '#', '#', sr)
            except Exception:  # file name most likely has 'invalid characters'. Create file as temp then rename.
                sr = arcpy.Describe(empty_file).spatialReference
                arcpy.CreateFeatureclass_management(gis_fol, 'temp.shp', geom_type, empty_file, '#', '#', sr)
                arcpy.Rename_management(os.path.join(gis_fol, 'temp.shp'), out_file)
            newlayer = arcpy.mapping.Layer(out_file)
            arcpy.mapping.AddLayer(data_frame, newlayer, "TOP")
            arcpy.RefreshActiveView()
            arcpy.RefreshTOC()
            return_message = "Successfully imported {0}".format(out_basename)
            return warning_status, return_message
    else:
        warning_status = True
        return_message = 'Invalid geometry type for {0}... skipping'.format(tf_type)
        return warning_status, return_message

# ______________________________________________________________________________________________________________________
# Main Code


if __name__ == "__main__":
    # Get arguments
    Points = arcpy.GetParameter(0)  # boolean
    Lines = arcpy.GetParameter(1)  # boolean
    Regions = arcpy.GetParameter(2)  # boolean
    tfTypes = arcpy.GetParameterAsText(3)
    OutputPrefix = arcpy.GetParameterAsText(4)
    EmptyFolder = arcpy.GetParameterAsText(5)
    
    # housekeeping on the input arguments
    if len(OutputPrefix) < 1:
        OutputPrefix = "Run_ID"  # provide a default value if unspecified
    if len(tfTypes) < 1:
        arcpy.AddError("\nNo TUFLOW empty file has been specified\n")
    if len(EmptyFolder) < 1:
        arcpy.AddError("\nTUFLOW Empty Folder must be specified\n")
    if not os.path.exists(EmptyFolder):
        arcpy.AddError("\nEmpty folder path does not exist\n{0}\n".format(EmptyFolder))
    
    # some arcmap stuff:
    mxd = arcpy.mapping.MapDocument("CURRENT")
    df = arcpy.mapping.ListDataFrames(mxd, "*")[0]
    
    # find gis folder
    GisFolder = EmptyFolder[:-6]  # removes last 6 characters (empty\)
    
    # Process: Feature Class To Shapefile (multiple)
    for tfType in tfTypes.split(';'):
        
        if Points:
            GeomType = "POINT"
            GeomExt = "_P"
            arcpy.AddMessage("\nProcessing... '{0}_{1}{2}.shp".format(tfType, OutputPrefix, GeomExt))
            WarningStatus, Message = import_empty(df, tfType, OutputPrefix, GeomType, GeomExt, GisFolder, EmptyFolder)
            if Warning:
                arcpy.AddWarning(Message)
            else:
                arcpy.AddMessage(Message)
    
        if Lines:
            GeomType = "POLYLINE"
            GeomExt = "_L"
            arcpy.AddMessage("\nProcessing... '{0}_{1}{2}.shp".format(tfType, OutputPrefix, GeomExt))
            WarningStatus, Message = import_empty(df, tfType, OutputPrefix, GeomType, GeomExt, GisFolder, EmptyFolder)
            if WarningStatus:
                arcpy.AddWarning(Message)
            else:
                arcpy.AddMessage(Message)
                
        if Regions:
            GeomType = "POLYGON"
            GeomExt = "_R"
            arcpy.AddMessage("\nProcessing... '{0}_{1}{2}.shp".format(tfType, OutputPrefix, GeomExt))
            WarningStatus, Message = import_empty(df, tfType, OutputPrefix, GeomType, GeomExt, GisFolder, EmptyFolder)
            if WarningStatus:
                arcpy.AddWarning(Message)
            else:
                arcpy.AddMessage(Message)
