# ---------------------------------------------------------------------------
# import_empties_v1.py
# Created on: 2012-03-30 17:00:27.00000
#   (generated by ArcGIS/ModelBuilder)
# Usage: import_empties_v1 <Points> <Lines> <Regions> <Empty_Shapefile> <Output_Prefix>
# Description:
# ---------------------------------------------------------------------------


import arcpy
import os       # required to check if files already exist
from arcpy import env
import sys
sys.path.append(r'C:\dev\python')
env.overwriteOutput = True
# Script arguments
#_______________________________________________________________________________
#ARGUMENTS
# debug
#_______________________________________________________________________________
##TF_type = '2d_zsh'
##Points = 'true'
##Lines = 'true'
##Regions = 'true'
##Output_suffix = 'test_001'
##config_filepath = r'C:\TUFLOW\ARC\arcTUFLOW\arcTUFLOW_testing_config.txt'
##ArcTUFLOW_dir ='C:\TUFLOW\ARC\arcTUFLOW'
#_______________________________________________________________________________
arcpy.AddMessage('Getting Arguments')
TF_type = arcpy.GetParameterAsText(0)
Points = arcpy.GetParameterAsText(1)
Lines = arcpy.GetParameterAsText(2)
Regions = arcpy.GetParameterAsText(3)
Output_suffix = arcpy.GetParameterAsText(4)
config_filepath = arcpy.GetParameterAsText(5) #optional, else it uses the default
ArcTUFLOW_dir = arcpy.GetParameterAsText(6) #optional only if not the default instalation directory

#_______________________________________________________________________________
# Default inputs
if config_filepath == '#' or not config_filepath:
    arcpy.AddMessage('Using default config file.')
    mxd = arcpy.mp.MapDocument("CURRENT")
    mxdpath = mxd.filePath
    arcpy.AddMessage('Map document = '+mxdpath)
    config_filepath = mxdpath[0:len(mxdpath)-4]+'_config.txt'
Layer_dir = r'C:\TUFLOW\ARC\arcTUFLOW\LayerFiles'
#_______________________________________________________________________________
# Some checks
arcpy.AddMessage("Checking " +config_filepath+" exists...")
if os.path.isfile(config_filepath):
    arcpy.AddMessage("Found.")
else:
    arcpy.AddMessage(config_filepath+" not found. Check Configure TUFLOW Database has been run.")

#_______________________________________________________________________________
# Display inputs
arcpy.AddMessage('Empty Type = '+TF_type)
arcpy.AddMessage('Points = '+Points)
arcpy.AddMessage('Lines = '+Lines)
arcpy.AddMessage('Regions = '+Regions)
arcpy.AddMessage('Run Id = '+Output_suffix)
arcpy.AddMessage('Configuration File = '+config_filepath)
arcpy.AddMessage('Location of arcTUFLOW install = '+ArcTUFLOW_dir)

#_______________________________________________________________________________
# Read Config
arcpy.AddMessage('Reading Config File.')
import arcTUFLOW_Config  # calls config module
config = arcTUFLOW_Config.read(config_filepath)
outgdb = config['gdb']
outlyrdir =config['lyr']
arcpy.AddMessage('Output Geodatabase = '+outgdb)
#_______________________________________________________________________________
# Default inputs
scriptPath = sys.path[0]
arcpy.AddMessage("Script folder: " + scriptPath)
toolSharePath = os.path.dirname(scriptPath)
arcpy.AddMessage("ToolShare folder: " + toolSharePath)
toolDataPath = os.path.join(toolSharePath, "ToolData")
arcpy.AddMessage("ToolData folder: " + toolDataPath)

Template_gdb = toolDataPath+'\\TUFLOW_Defaults.gdb'
arcpy.AddMessage('Extracting Feature Classes from '+Template_gdb)

# some arcmap stuff:
arcpy.AddMessage('Getting Mapdocument and data frame')
mxd = arcpy.mp.MapDocument("CURRENT")
df = arcpy.mp.ListDataFrames(mxd,"*")[0]

# input feature classes from template gdb
arcpy.AddMessage('Setting Empties')
fcname = '_'+TF_type+'_empty_'
outdesc = TF_type+'_'+Output_suffix
infcP = Template_gdb+'\\Templates\\'+fcname+'P'
infcL = Template_gdb+'\\Templates\\'+fcname+'L'
infcR = Template_gdb+'\\Templates\\'+fcname+'R'
arcpy.AddMessage('ptsfc = '+infcP)
inlyrP = Layer_dir+'\\'+fcname+'P.lyr'
inlyrL = Layer_dir+'\\'+fcname+'L.lyr'
inlyrR = Layer_dir+'\\'+fcname+'R.lyr'

# setting outputs
arcpy.AddMessage('Setting Outputs')
outfcP = outgdb+'\\_'+TF_type+'_'+Output_suffix+'_P'
outfcL = outgdb+'\\_'+TF_type+'_'+Output_suffix+'_L'
outfcR = outgdb+'\\_'+TF_type+'_'+Output_suffix+'_R'
outlyrP = outlyrdir+'\\'+TF_type+'_'+Output_suffix+'_P.lyr'
outlyrL = outlyrdir+'\\'+TF_type+'_'+Output_suffix+'_L.lyr'
outlyrR = outlyrdir+'\\'+TF_type+'_'+Output_suffix+'_P.lyr'

if Points == 'true':
    try:
        # create layer
        arcpy.AddMessage('Creating Points from template.')
        arcpy.AddMessage('In Feature Class = '+ infcP)
        arcpy.AddMessage('Out Feature Class = '+ outfcP)
        arcpy.AddMessage('In Layerfile= '+ inlyrP)
        arcpy.AddMessage('Out Layerfile = '+ outlyrP)
        arcpy.AddMessage('Creating point feature class.')
        arcpy.CopyFeatures_management(infcP,outfcP)
        arcpy.AddMessage('Creating layer.')
        arcpy.MakeFeatureLayer_management(outfcP, outdesc+'_P')
        arcpy.AddMessage('Saving to layer file to '+outlyrL+'.')
        arcpy.SaveToLayerFile_management(outdesc+'_P', outlyrL, 'ABSOLUTE')
        arcpy.AddMessage('Applying symbology')
        arcpy.ApplySymbologyFromLayer_management (outlyrL, inlyrP)
        arcpy.AddMessage('Adding Layer to current dataframe.')
        addLayer = arcpy.mp.Layer(outlyrL)
        arcpy.AddMessage('Add layer = '+addLayer)
        arcpy.mp.AddLayer(df, addLayer,"TOP")
        msg = arcpy.GetMessages()
        arcpy.AddMessage(msg)
        #arcpy.RefreshActiveView()
        #arcpy.RefreshTOC()
        del addLayer
        arcpy.AddMessage('Done.')
    except:
        msg = arcpy.GetMessages()
        arcpy.AddMessage(msg)


##if Lines == 'true':
##    try:
##        # create layer
##        arcpy.AddMessage('Creating Lines from template.')
##        arcpy.CopyFeatures_management(linfc,outlin)
##        arcpy.AddMessage('Done.')
##        newlayer = arcpy.mp.Layer(outlin)
##        arcpy.mp.AddLayer(df, newlayer,"TOP")
##        arcpy.RefreshActiveView()
##        arcpy.RefreshTOC()
##        del newlayer
##    except:
##        msg = arcpy.GetMessages()
##        arcpy.AddMessage(msg)

##if Regions == 'true':
##    try:
##        # create layer
##        arcpy.AddMessage('Creating Regions from template.')
##        arcpy.CopyFeatures_management(regfc,outreg)
##        arcpy.AddMessage('Done.')
##    except:
##        msg = arcpy.GetMessages()
##        arcpy.AddMessage(msg)

