# ---------------------------------------------------------------------------
# import_empties_v1.py
# Created on: 2012-03-30 17:00:27.00000
#   (generated by ArcGIS/ModelBuilder)
# Usage: import_empties_v1 <Points> <Lines> <Regions> <Empty_Shapefile> <Output_Prefix> 
# Description: 
# ---------------------------------------------------------------------------


import arcpy
import os       # required to check if files already exist

# Script arguments
Points = arcpy.GetParameterAsText(0)
if Points == '#' or not Points:
    Points = False # provide a default value if unspecified

if Points:
	arcpy.AddMessage("Processing points")

Lines = arcpy.GetParameterAsText(1)
if Lines == '#' or not Lines:
    Lines = False # provide a default value if unspecified
if Lines:
	arcpy.AddMessage("Processing lines")

Regions = arcpy.GetParameterAsText(2)
if Regions == '#' or not Regions:
    Regions = False # provide a default value if unspecified
if Regions:
	arcpy.AddMessage("Processing regions")

tf_types = arcpy.GetParameterAsText(3)
for tf_type in tf_types.split(';'):
	arcpy.AddMessage("TUFLOW type: "+tf_type)

Output_Prefix = arcpy.GetParameterAsText(4)
if Output_Prefix == '#' or not Output_Prefix:
    Output_Prefix = "Run_ID" # provide a default value if unspecified
arcpy.AddMessage("Run ID: "+Output_Prefix)
	
empty_folder = arcpy.GetParameterAsText(5)
if empty_folder == '#' or not empty_folder:
    arcpy.AddError("TUFLOW Empty Folder must be specified.")
else:
	arcpy.AddMessage("TUFLOW empty folder: "+empty_folder)


# some arcmap stuff:
mxd = arcpy.mp.MapDocument("CURRENT")
df = arcpy.mp.ListDataFrames(mxd,"*")[0]

# find gis folder
gis_fol = empty_folder[0:len(empty_folder)-5] # removes last 6 characters (empty\)
arcpy.AddMessage("Output folder: "+gis_fol)


# Process: Feature Class To Shapefile (multiple)
arcpy.AddMessage("Looping through specied input types...")
for tf_type in tf_types.split(';'):
	arcpy.AddMessage("TUFLOW type: "+tf_type)
	ptspath = empty_folder+"\\"+tf_type+"_empty_P.shp"
	linpath = empty_folder+"\\"+tf_type+"_empty_L.shp"
	regpath = empty_folder+"\\"+tf_type+"_empty_R.shp"
	if (Points): #process
		arcpy.AddMessage("Processing Points")
		if (os.path.isfile(ptspath)):
			arcpy.AddMessage("Processing "+ptspath)
			outpts = gis_fol+tf_type+'_'+Output_Prefix+'_P.shp'
			prjfile = empty_folder+"\\"+tf_type+"_empty_P.prj"
			spatial_reference = arcpy.Describe(ptspath).spatialReference
			out_shp = tf_type+'_'+Output_Prefix+'_P.shp'
			arcpy.AddMessage("Output File: "+outpts)
			if os.path.isfile(outpts):
				arcpy.AddWarning('File '+outpts+' already exists ...skipping')
			else:
				#arcpy.FeatureClassToFeatureClass_conversion(ptspath, gis_fol,out_shp)
				#arcpy.CopyFeatures_management(ptspath,outpts)
				arcpy.CreateFeatureclass_management(gis_fol, out_shp, "POINT", ptspath, "DISABLED", "DISABLED", spatial_reference)
				try:
					arcpy.arcpy.DefineProjection_management(outpts, prjfile)
				except:
					arcpy.AddWarning('Issue setting coordinate system of file.')
				newlayer = arcpy.mp.Layer(outpts)
				arcpy.mp.AddLayer(df, newlayer,"TOP")
				arcpy.RefreshActiveView()
				arcpy.RefreshTOC()
				del newlayer
				
		else:
			print 'Geometry type points not available for '+tf_type+' ...skipping'

	if (Lines): #process
		arcpy.AddMessage("Processing Lines")
		if (os.path.isfile(linpath)):
			arcpy.AddMessage("Processing "+linpath)
			outlin = gis_fol+tf_type+'_'+Output_Prefix+'_L.shp'
			prjfile = empty_folder+"\\"+tf_type+"_empty_L.prj"
			spatial_reference = arcpy.Describe(linpath).spatialReference
			out_shp = tf_type+'_'+Output_Prefix+'_L.shp'
			arcpy.AddMessage("Output File: "+outlin)
			if os.path.isfile(outlin):
				arcpy.AddWarning('File '+outlin+' already exists ...skipping')
			else:
				arcpy.CreateFeatureclass_management(gis_fol, out_shp, "POLYLINE", linpath, "DISABLED", "DISABLED", spatial_reference)
				try:
					arcpy.arcpy.DefineProjection_management(outlin, prjfile)
				except:
					arcpy.AddWarning('Issue setting coordinate system of file.')
				newlayer = arcpy.mp.Layer(outlin)
				arcpy.mp.AddLayer(df, newlayer,"TOP")
				arcpy.RefreshActiveView()
				arcpy.RefreshTOC()
				del newlayer
				
		else:
			print 'Geometry type lines not available for '+tf_type+' ...skipping'
			
	if (Regions): #process
		arcpy.AddMessage("Processing Regions")
		if (os.path.isfile(regpath)):
			arcpy.AddMessage("Processing "+regpath)
			outreg = gis_fol+tf_type+'_'+Output_Prefix+'_R.shp'
			prjfile = empty_folder+"\\"+tf_type+"_empty_R.prj"
			spatial_reference = arcpy.Describe(regpath).spatialReference
			out_shp = tf_type+'_'+Output_Prefix+'_R.shp'
			arcpy.AddMessage("Output File: "+outreg)
			if os.path.isfile(outreg):
				arcpy.AddWarning('File '+outreg+' already exists ...skipping')
			else:
				arcpy.CreateFeatureclass_management(gis_fol, out_shp, "POLYGON", regpath, "DISABLED", "DISABLED", spatial_reference)
				try:
					arcpy.arcpy.DefineProjection_management(outreg, prjfile)
				except:
					arcpy.AddWarning('Issue setting coordinate system of file.')
				newlayer = arcpy.mp.Layer(outreg)
				arcpy.mp.AddLayer(df, newlayer,"TOP")
				arcpy.RefreshActiveView()
				arcpy.RefreshTOC()
				del newlayer
				
		else:
			print 'Geometry type lines not available for '+tf_type+' ...skipping'
arcpy.AddMessage("Import successful")
