'''
Load Simulation Files from TLF File

version 1.0

author: Wood Rodgers, Inc.
	http://www.woodrodgers.com
Check file modifications by Mitchell.Smith@bmtwbm.com.au 03/09/2015

'''
import arcview # Options: arcview, arceditor, arcinfo
import arcpy
import os, string, sys, time, math, traceback, fileinput, linecache,glob
import csv, re

yyyy = time.strftime("%Y", time.localtime())
mm = time.strftime("%m", time.localtime())
dd = time.strftime("%d", time.localtime())
map_doc = "CURRENT"
in_tuflow = sys.argv[1]
in_tlf = sys.argv[2]
on_layers = sys.argv[3]
walk = sys.argv[4]

def main():

    # Check inputs
    if not os.path.isfile(in_tlf):
        arcpy.AddMessage('There is an issue with your input tlf. Please check.')
        sys.exit()

    dir_path = os.path.dirname(os.path.realpath(__file__))
    lyrPath = os.path.join(dir_path, '..\ToolData\LayerFiles\Check')
    tuflow_GroupLayer = os.path.join(dir_path,'..\ToolData\LayerFiles\Check\TUFLOW_(EmptyGroup).lyr')

    # Check that the directories exist
    if not os.path.exists(lyrPath):
        arcpy.AddMessage('Layer file directory doesn''t exist. Please ensure that your directory ..\ToolData\LayerFiles\Check exists.')
        sys.exit()
    if not os.path.isfile(tuflow_GroupLayer):
        arcpy.AddMessage('Empty Group Symbology doesn''t exist. Please check')
        sys.exit()

    # get the check file wildcard from the tlf
    pp,ff = os.path.split(in_tlf)
    wild_card = ff[:-4]

	# load available layer files.
    lyr_files = glob.glob(lyrPath + '\*.lyr')

    #set up mxd
    arcpy.env.workspace = os.path.dirname(map_doc)
    mxd = arcpy.mp.MapDocument(map_doc)
    df = arcpy.mp.ListDataFrames(mxd,"*")[0]   # Need to make this generic based on current data frame
    groupLayer = arcpy.mp.Layer(tuflow_GroupLayer)
    arcpy.mp.AddLayer(df, groupLayer,"TOP")
    targetGroupLayer = arcpy.mp.ListLayers(mxd, "TUFLOW", df)[0]
    layer_list = []

    if walk == 'true':
        check_files = []
        for dirpath, dirnames, files in os.walk(in_tuflow):
            for d in dirnames:
                tmp = glob.glob(in_tuflow + '\\' + d + '\*'+ wild_card +'*.shp')
                check_files = check_files + tmp
    else:
        check_files = glob.glob(in_tuflow + '\*'+ wild_card +'*.shp')

    #arcpy.AddMessage(check_files)
    arcpy.AddMessage(walk)

    # Remove the directory
    lyr_name = []
    for ly in lyr_files:
        pp,ff = os.path.split(ly)
        pp,ff = ff.split('.')
        lyr_name.append(pp)

    if not check_files:
        arcpy.AddMessage( "No check files found for this run")
        sys.exit()

	# Add the check layers to arcmap.
    for chf in check_files:
        addLayer = arcpy.mp.Layer(chf)
        arcpy.mp.AddLayerToGroup(df, targetGroupLayer, addLayer, "BOTTOM")
        # Switch the layer off by default
        if not on_layers == 'true':
            for lyr in arcpy.mp.ListLayers(mxd):
                lyr.visible = False
            arcpy.RefreshTOC()
            arcpy.RefreshActiveView()
        # Switch off zpt, uvpt and grd
        for lyr in arcpy.mp.ListLayers(mxd):
            if any(substring in lyr.name for substring in ['grd','uvpt','zpt']):
                if not any(substring in lyr.name for substring in ['_zsh_zpt_','_zln_zpt_']):  # Don't turn off the zsh or zln layers.
                    lyr.visible = False

    #apply symbology to layers in mxd
    lf_list = os.listdir(lyrPath)
    for lyr in arcpy.mp.ListLayers(mxd, "TUFLOW", df):
        if lyr.isGroupLayer:
            for slyr in lyr:
                try:
                    # Get the layer name in a format suitable for string comparison
                    tmp =slyr.name.split('\\')
                    tmp = str(tmp[0])

                    # Which layer file do we grab
                    lyrFile= [s for s in lyr_name if s in tmp]
                    lyrFile =str(lyrFile[0])
                    #arcpy.AddMessage(lyrFile) # Debugging output the layer name

                    # Check that _zpt_check is not within _zsh_zpt_check or _zln_zpt_check
                    if '_zsh_zpt_check_P' in tmp:
                        lyrFile = '_zsh_zpt_check_P'
                        #arcpy.AddMessage('_zsh_zpt_check updated')
                    elif '_zln_zpt_check_P' in tmp:
                        lyrFile = '_zln_zpt_check_P'
                        #arcpy.AddMessage('_zln_zpt_check updated')

                    if lyrFile:
                        arcpy.AddMessage('lyr file for selection is: ' + lyrFile)
                    else:
                        arcpy.AddMessage('No layer file has been selected')

                    lyrFile =  "{0}.lyr".format(lyrFile)

                    #arcpy.AddMessage(lyrFile)
                    if lyrFile in lf_list:
                        updateLayer = arcpy.mp.ListLayers(mxd, slyr.datasetName, df)[0]
                        sourceLayer = arcpy.mp.Layer(os.path.join(lyrPath, lyrFile))
                        arcpy.mp.UpdateLayer(df, updateLayer, sourceLayer, True)

                        if lyrFile == '_bcc_check_R.lyr':
                            # Calculate the symbology
                            if slyr.symbologyType == "UNIQUE_VALUES":
                                slyr.symbology.valueField = "Source"
                                slyr.symbology.addAllValues()

                        elif lyrFile == '_1d_to_2d_check_R.lyr' or lyrFile == '_2d_to_2d_R.lyr':
                            if slyr.symbologyType == "UNIQUE_VALUES":
                                arcpy.AddMessage('Updating Symbology')
                                time.sleep(2)
                                slyr.symbology.valueField = "Primary_No"
                                slyr.symbology.addAllValues()

                        elif lyrFile == '_sac_check_R.lyr':
                            if slyr.symbologyType == "UNIQUE_VALUES":
                                arcpy.AddMessage('Updating Symbology')
                                time.sleep(2)
                                slyr.symbology.valueField = "BC_Name"
                                slyr.symbology.addAllValues()
                except:
                    arcpy.AddMessage('Could not find Layer File for ' + slyr.name)
                    continue

if __name__ == '__main__':
    main()
